/**
 * SPDX-FileCopyrightText: 2024 Sascha Nitsch (grumpydeveloper) https://contentnation.net/@grumpydevelop
 * SPDX-License-Identifier: GPL-3.0-or-later
 * @author Author: Sascha Nitsch (grumpydeveloper)
 * @internal
 **/

import { RTNode } from './rtnode';
import { RenderTree } from './rendertree';

/**
 * Text node
 */
export class RTText extends RTNode {
  /** left position of text box */
  private x0: string;
  /** top position of text box */
  private y0: string;
  /** rigth position of text box */
  private x1: string;
  /** left bottom of text box */
  private y1: string;
  /** width of text box */
  private width: string;
  /** height of text box */
  private height: string;
  /** compact list of [left, top, right, bottom] */
  private pos: string;
  /** horizontal alignment of text in box */
  private halign: string;
  /** vertical alignment of text in box */
  private valign: string;
  /** font size */
  private fontSize: string;
  /** actual text */
  private text: string;
  /** text color */
  private color: string;

  /**
   * clone function
   * @return a copy of us
   */
  clone(): RTText {
    return new RTText(
      this.root,
      this.x0,
      this.y0,
      this.x1,
      this.y1,
      this.width,
      this.height,
      this.pos,
      this.halign,
      this.valign,
      this.fontSize,
      this.text,
      this.color
    );
  }

  /**
   * create Node from HTMLElement
   * @internal
   * */
  static fromElem(root: RenderTree, node: HTMLElement): RTText {
    return new RTText(
      root,
      node.getAttribute('x0'),
      node.getAttribute('y0'),
      node.getAttribute('x1'),
      node.getAttribute('y1'),
      node.getAttribute('w'),
      node.getAttribute('h'),
      node.getAttribute('pos'),
      node.getAttribute('halign') || 'center',
      node.getAttribute('valign'),
      node.getAttribute('size'),
      node.innerHTML,
      node.getAttribute('color') || root.getDefinition('defaultfontcolor')
    );
  }

  /**
   * constructor
   * @param root parent RenderTree
   * @param x0 left position of text box
   * @param y0 top position of text box
   * @param width width of text box
   * @param height heightof text box
   * @param pos short version of [left, top, right, bottom]
   * @param halign horizontal alignment in text box
   * @param valign vertical alignment in text box
   * @param fontSize font size
   * @param text actural text
   * @param color text color
   */
  constructor(root: RenderTree, x0: string, y0: string, x1: string, y1: string, width: string, height: string, pos: string, halign: string, valign: string, fontSize: string, text: string, color: string) {
    super(root);
    this.x0 = x0;
    this.y0 = y0;
    this.x1 = x1;
    this.y1 = y1;
    this.width = width;
    this.height = height;
    this.pos = pos;
    this.halign = halign;
    this.valign = valign;
    this.fontSize = fontSize;
    this.text = text;
    this.color = color;
  }

  /**
   * render image to HTML target
   * @param target target to render to
   */
  render(target: HTMLElement): void {
    let x0: number;
    let y0: number;
    let x1: number;
    let y1: number;
    if (this.pos) {
      const posLookup = this.resolveString(this.pos);
      const tokens = posLookup.split(',');
      if (tokens.length != 4) {
        return;
      }
      x0 = this.calc(this.resolveString(tokens[0]));
      y0 = this.calc(this.resolveString(tokens[1]));
      x1 = this.calc(this.resolveString(tokens[2]));
      y1 = this.calc(this.resolveString(tokens[3]));
    } else {
      x0 = this.calc(this.resolveString(this.x0));
      y0 = this.calc(this.resolveString(this.y0));
      if (this.x1) {
        x1 = this.calc(this.resolveString(this.x1));
        y1 = this.calc(this.resolveString(this.y1));
      } else {
        x1 = x0 + this.calc(this.resolveString(this.width));
        y1 = y0 + this.calc(this.resolveString(this.height));
      }
    }
    const halign = this.resolveString(this.halign);
    const valign = this.resolveString(this.valign);
    const text = this.resolveString(this.text);
    const div = document.createElement('div');
    if (this.color) {
      div.style.color = this.resolveString(this.color);
    }
    div.style.position = 'absolute';
    if (this.fontSize) {
      div.style.fontSize = this.calc(this.resolveString(this.fontSize)) + 'px';
    }
    div.style.left = x0 + 'px';
    div.style.top = y0 + 'px';
    div.style.width = (x1 - x0) + 'px';
    div.style.height = (y1 - y0) + 'px';
    div.style.overflow = 'clip';
    switch (halign) {
      case 'center':
        div.style.textAlign = 'center';
        break;
      case 'right':
        div.style.textAlign = 'right';
        break;
    }
    switch (valign) {
      case 'center':
        div.style.alignContent = 'center';
        break;
      case 'bottom':
        div.style.alignContent = 'end';
        break;
    }
    div.insertAdjacentHTML('afterbegin', text);
    target.append(div);
  }
}
